/* Program:	COMSPEC.EXE
   Programmer:	Kenneth I McKinlay
   Syntax:	COMSPEC
   Returns:	0: Successful
		1: Error executing
		2: Running under Windows
   Version:	1.1 - Checks for Windows
		1.0 - Initially release version
   Description:	The function of this program is to re-order the master
		environment table of a computer in order to place the
		COMSPEC variable at the top of the environment list.  This
		program was written to bypass a problem with another system
		that required COMSPEC at the start of the environment.
   Disclaimer:	This program is released into the Public Domain.  Kenneth
		I McKinlay can not be held responsible for any problems due
		to this program.  In plain english...USE AT OWN RISK.  This
		program has been tested extensively but no guarantees are
		made.
   E-Mail:	If you have any questions or problem reports, I can be
		contacted by E-Mail via CompuServe at 71531,3150.
*/

#include <dos.h>
#include <malloc.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern	int	IsWindows(void);
void		HelpScreen(void);

char far	*Mstr_getenv(char far *, char far *);
void		Mstr_FindEnvironment(char far **, unsigned *);

static	char far	comspecString[80];

void main(int argc)
{
	char far	*masterEnvironmentPtr;
	char far	*comspecEnvirPtr;
	int		comspecOffset,comspecLength;
	unsigned	envSize;

	/* Check if a parameter is specified */
	argc--;
	if (argc)
		HelpScreen();

	/* Check if Microsoft Windows is running, if so, exit.  It can
	   cause the system to hang if run from with-in a DOS box. */
	if (IsWindows())
	{
		printf("Cannot run under Microsoft Windows.\n");
		exit(2);
	}

	/* Locate the master environment table */
	Mstr_FindEnvironment(&masterEnvironmentPtr,&envSize);

	/* Find the COMSPEC string in the environment */
	if ((comspecEnvirPtr=Mstr_getenv(masterEnvironmentPtr,"COMSPEC=")) == NULL)
	{
		printf("Unable to find COMSPEC.\n");
		exit(1);
	}

	/* Determine the offset of the COMSPEC from the top of environment */
	comspecOffset=comspecEnvirPtr-masterEnvironmentPtr;
	if (comspecOffset > 0)
	{
		/* Store the value for COMSPEC in a local variable */
		memmove(comspecString,comspecEnvirPtr,80);
		comspecString[80]='\0';
		comspecLength=strlen(comspecString)+1;

		/* Reorder to enviroment to block out COMSPEC */
		memmove(masterEnvironmentPtr+comspecLength,masterEnvironmentPtr,comspecOffset);
		/* Add COMSPEC to the top of the environment */
		memmove(masterEnvironmentPtr,comspecString,comspecLength);
	}

	/* Exit to DOS with errorlevel set to 0 */
	exit(0);
}

/*
 *   Mstr_FindEnvironment:
 *     Scans for the "Master" Environment area, and returns
 *     a pointer to it, and the size of the environment.
*/
void Mstr_FindEnvironment ( char far **Env , unsigned *EnvSize )
{
   unsigned int far *CommandSeg, far *TempSeg ;
   char far *BlockSeg ;

   /*
    *  Scan through PSP's looking for a block that is its own father.
    *  This block is the PSP of COMMAND.COM
   */
  TempSeg = (unsigned int far *)(_psp*0x10000); /* Convert to a far pointer, replaces MK_FP */
   do
   {
     CommandSeg = TempSeg ;
     TempSeg = (unsigned int far *)(*(TempSeg+8)*0x10000); /* Replaces MK_FP in turbo C */
   }
   while ( TempSeg != CommandSeg ) ;

   /*
    *  Scan forward through memory looking for the correct MSB.
    *  This will have COMMAND.COM's PSP as owner, and begin with
    *  the character M
   */
   BlockSeg = (char far *)CommandSeg ;
   do
   {
    BlockSeg = (char far *)((FP_SEG(BlockSeg)+1)*0x10000); /* Replaces MK_FP */
   }
   while ( ( *(unsigned int far *)(BlockSeg+1) != FP_SEG ( CommandSeg ) ) ||
           ( *BlockSeg != 'M' ) ) ;

   /*
    *  The environment is the NEXT segment of memory
    *  and bytes 4 and 5 are the size in paragraphs
   */
   *Env = (char far *)((FP_SEG(BlockSeg)+1)*0x10000); /* Replaces MK_FP */
   *EnvSize = 16 * *(unsigned int far *)(BlockSeg+3) ;
}

/*
 *   Mstr_getenv:
 *     Scans the "Master" Environment for a given "sub string"
 *     and returns a pointer to it.
 *     Similar to Turbo routine "getenv" but uses the Master copy of the
 *     environment table.
*/
char far *Mstr_getenv (char far *Env , char far *name)
{
   char far *Sub_Env, far *str1, far *str2 ;

   /*
    *  Start at the beginning of the environment
   */
   Sub_Env = Env ;

   /*
    *  While the "sub string" we're looking at is non-zero
   */
   for ( ; *Sub_Env ; )
   {
     /*
      *  Simulate a "strcmp" on the "sub string" of the environment
      *  and the string we're looking for
     */
     for ( str1 = Sub_Env , str2 = name ;
           (*str1) && (*str2) && ( *str1 == *str2) ;
           str1++ , str2++ ) ;
     /*
      *  If we reached the end of the string we're looing for
      *  we've found the correct portion of the environment.
      *  Return the ptr to the start of this "sub string"
     */
     if ( !*str2 )
       return ( Sub_Env ) ;

     /*
      *  Otherwise, advance to the next "sub string" in the environment
      *  by performing a "strchr" function
     */
     for ( ; *(Sub_Env++) ; ) ;
   }

   /*
    *  Obviously, the string is not present in the environment.
    *  Return this fact.
   */
  return ( NULL ) ;
}

void HelpScreen()
{
	printf("\nComspec  v1.1  (c) 1993 Kenneth I McKinlay\n");
	printf("           Moves the COMSPEC variable to the top of the environment.\n");
	exit(1);
}
