/* Bill's 'Message'

	Version 1.0 (16 October 1993 first release)

	Written out of necessity for displaying a message to users
	when they log on to the network. The program automatically
	terminates after a fixed delay, as some network user accounts
	must not be allowed to pause continuously during normal log in.

	Usage:

	Tested on PC-AT computers with colour VGA display.

	Needs Borland C++ v3.0 compiler with small memory model.

	Modifications:

	1.0   14th November:

			/dv = Direct Video RAM support added as an speed enhancement
			option.

			Oops!  Failed to pass MESSAGE environment to MESSAGE.TXT
			in read_text()

			MESSAGE environment variable no longer supported. Message
			filename now specified at the command line.

*/


#include <alloc.h>
#include <conio.h>
#include <dir.h>
#include <dos.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>



#define LEFT_X 2
#define LEFT_Y 2
#define RIGHT_X 78
#define RIGHT_Y 23
#define MARGIN_X 3
#define MARGIN_Y 1
#define INITIAL_WAIT_TIME 20
#define EXTENDED_WAIT_TIME 30



main(argc,argv)
int argc;
char *argv[];
{
	FILE *stream;
	int drive_no;
	int max_drive;
	int count;
	int terminate = 0;
	char *path, *path_ptr;
	extern int directvideo;


	/* set video access method, BIOS call */
	directvideo=0;

	/* read filename of message file */
	path = (char *) malloc(strlen(argv[1])+3);
	strcpy(path,argv[1]);


	/* read command-line parameters */
	for(count=0; count<argc; ++count)
		{
		/* display help screen */
		if(strcmp(argv[count],"/?")==0)
			{
			show_help();
			getch();
			}

		/* write directly to video RAM */
		if(strcmpi(argv[count],"/dv")==0)
			{
			directvideo = 1;
			}
		}


	/* check for presence of message file */
	if((stream = fopen(path, "r")) == NULL)
		{
		/* no message file found */
		terminate = 1;
		}
	close(stream);

	/* display message if found */
	if(terminate == 0)
		{
		clear_screen();

		messagebox();
		read_text_file(path);
		count_down();

		clear_screen();
		}

	return(terminate);
}


int messagebox()
{
	int temp;
	int left_x=LEFT_X, left_y=LEFT_Y, right_x=RIGHT_X, right_y=RIGHT_Y;

	textbackground(LIGHTGRAY);
	textcolor(BLACK);
	window(left_x,left_y,right_x,right_y);
	clrscr();

	textcolor(BLACK);
	cprintf("\r ");
	textcolor(WHITE);
	cprintf("\r\n");
	for(temp=1;temp<=right_y-left_y-1;temp++)
		{
		textcolor(BLACK);
		cprintf("                                                                          ");
		textcolor(WHITE);
		cprintf("\r\n");
		}
	textcolor(BLACK);
	cprintf(" ");
	textcolor(WHITE);
	cprintf("ͼ");

	return 0;

}


int clear_screen()
{
	/* clears text screen */
	window(1,1,80,25);
	textbackground(BLACK);
	textcolor(LIGHTGRAY);
	clrscr();

	return 0;
}


int read_text_file(path)
char *path;
{
	FILE *stream;
	char ch;
	int char_count, line_number;

	textbackground(LIGHTGRAY);
	window(LEFT_X+MARGIN_X,LEFT_Y+MARGIN_Y,RIGHT_X-MARGIN_X,RIGHT_Y-MARGIN_Y);

	/* open message file for reading */
	if((stream = fopen(path, "r")) != NULL)
		{

		/* this variable will be used for checking line length */
		char_count=0;

		/* this variable will track how many lines are to be printed */
		line_number=1;

		/* get and print text */
		textcolor(BLUE);
		do
			{
			/* read a char from the file */
			ch = fgetc(stream);

			/* inserts carriage return when end of line detected */
			if (ch == 10)
				{
				cprintf("\r");
				char_count=0;
				line_number++;
				}

			/* record length of current line */
			char_count++;

			/* character is only to be printed if line length not exceeded */
			if((char_count < (RIGHT_X-LEFT_X-2*(MARGIN_X))) && (line_number<RIGHT_Y-LEFT_Y-2*(MARGIN_Y)-2))
				{
				/* display the character */
				cprintf("%c",ch);
				}
			if (line_number == 2) textcolor(BLACK);
			} while (ch != EOF);
		}
	fclose(stream);
	return 0;

}




int count_down()
{
	clock_t start, present, end;
	int response = 0;
	int key_pressed;
	int wait_time = INITIAL_WAIT_TIME;
	int seconds_gone;

	start = clock();

	textcolor(RED);
	cprintf("\n\n\rPress 'P' to continue viewing, or any other key to log in.\n");

	textcolor(BLUE);

	while(response == 0)
		{
		while((response == 0))
			{
			/* 0.25 second delay */
			delay(250);
			present = clock();
			seconds_gone = (present - start)/ CLK_TCK;

			/* quit program if time expires */
			if (seconds_gone > wait_time) response = 1;

			/* show time remaining */
			cprintf("\rTime remaining = %d secs.  ", (wait_time) - seconds_gone);
			if(kbhit())
				{
				response=1;
				key_pressed = getch();
				}
			}

		/* if P or p is pressed, extend timeout delay */
		if((key_pressed==112) || (key_pressed==80))
			{
			wait_time=EXTENDED_WAIT_TIME - 1;
			response=0;
			start = clock();
			key_pressed=0;
			}
		}
		return 0;
}

int show_help()
{
		printf("\n\nMessage (c) Bill LEUNG, 100275,3666@Compuserve.com\n");
		printf("--------------------------------------------------\n");
		printf("(v1.0, 14 Nov 1993)\n\n");
		printf("Displays a message on screen for 20 or more seconds.\n\n");
		printf("Usage:  MESSAGE filename [/dv]\n\n");
		printf("          filename -  specifies the text file to displayed.\n");
		printf("          /dv      -  write directly to Video memory.\n\n");
		printf("        This program may be freely distributed. There is no\n");
		printf("        charge for using this program.\n\n");
		printf("Press any key to continue.\n");

		return 0;
}